%Solve Laplace Equation by ADI
%Solve 2D Laplace Equation by ADI method

function Lap_sol = ADI_liquid(alpha, beta, gamma, lama, lamc, co_xy, phi_appl, Lap_ini, m, n, m_e, it_max, err)
    Lap_old = Lap_ini;
    Lap_half = zeros(m,n);
    Lap_new = zeros(m,n);
    co_al_b = zeros(1,n);
    co_be_b = zeros(1,n);
    co_ga_b = zeros(1,n);
    co_de_b = zeros(1,n);
    co_al_a = zeros(1,m);
    co_be_a = zeros(1,m);
    co_ga_a = zeros(1,m);
    co_de_a = zeros(1,m);
    Lap_sol = zeros(m,n);
    
    %Start ADI
    for C_it = 1:it_max
        %a implicit, b explicit
        for j = 2:(n-1)
            %symmetry boundary
            co_al_a(1) = 0;
            co_be_a(1) = 1;
            co_ga_a(1) = -1;
            co_de_a(1) = 0;
            for i = 2:(m-1)
                co_al_a(i) = alpha(i,j);
                co_be_a(i) = beta(i,j);
                co_ga_a(i) = gamma(i,j);
                co_de_a(i) = -lama(i,j) * Lap_old(i,j-1) - lamc(i,j) * Lap_old(i,j+1) - co_xy(i,j) * (Lap_old(i+1,j+1) - Lap_old(i+1,j-1) - Lap_old(i-1,j+1) + Lap_old(i-1,j-1));
            end
            %right boundary
            co_al_a(m) = 0;
            co_be_a(m) = 1;
            co_ga_a(m) = 0;
            co_de_a(m) = 0;
            C_sol = Thomas(co_al_a, co_be_a, co_ga_a, co_de_a, m);
            for i = 1:m
                Lap_half(i,j) = C_sol(i);
            end
        end
        for i = 1:m_e  %Apply boundary condition for electrolyte, j = 1 and j = n
            Lap_half(i,1) = 0;  %phi(i,1) = 0
            Lap_half(i,n) = 1;  %phi(i,n) = 1
        end
        for i = (m_e+1):m  %Apply boundary condition for shield, j = 1 and j = n
            Lap_half(i,1) = 0;  %phi(i,1) = 0
            Lap_half(i,n) = Lap_half(i,n-1);  %phi(i,n) = phi(i,n-1);
        end
        
        %b implicit, a explicit
        %electrolyte
        for i = 2:m_e
            co_al_b(1) = 0;
            co_be_b(1) = 1;
            co_ga_b(1) = 0;
            co_de_b(1) = 0;
            for j = 2:(n-1)
                co_al_b(j) = lama(i,j);
                co_be_b(j) = beta(i,j);
                co_ga_b(j) = lamc(i,j);
            	co_de_b(j) = -alpha(i,j) * Lap_half(i-1,j) - gamma(i,j) * Lap_half(i+1,j) - co_xy(i,j) * (Lap_half(i+1,j+1) - Lap_half(i+1,j-1) - Lap_half(i-1,j+1) + Lap_half(i-1,j-1));
            end
            co_al_b(n) = 0;
            co_be_b(n) = 1;
            co_ga_b(n) = 0;
            co_de_b(n) = phi_appl;
            C_sol = Thomas(co_al_b, co_be_b, co_ga_b, co_de_b, n);
            for j = 1:n
            	Lap_new(i,j) = C_sol(j);
            end
        end
        
        %shield
        for i = (m_e+1):(m-1)
            co_al_b(1) = 0;
            co_be_b(1) = 1;
            co_ga_b(1) = 0;
            co_de_b(1) = 0;
            for j = 2:(n-1)
                co_al_b(j) = lama(i,j);
                co_be_b(j) = beta(i,j);
                co_ga_b(j) = lamc(i,j);
            	co_de_b(j) = -alpha(i,j) * Lap_half(i-1,j) - gamma(i,j) * Lap_half(i+1,j) - co_xy(i,j) * (Lap_half(i+1,j+1) - Lap_half(i+1,j-1) - Lap_half(i-1,j+1) + Lap_half(i-1,j-1));
            end
            co_al_b(n) = -1;
            co_be_b(n) = 1;
            co_ga_b(n) = 0;
            co_de_b(n) = 0;
            C_sol = Thomas(co_al_b, co_be_b, co_ga_b, co_de_b, n);
            for j = 1:n
            	Lap_new(i,j) = C_sol(j);
            end
        end
        for j = 1:n  %Apply boundary condition for i = 1 and i = m
            Lap_new(1,j) = Lap_new(2,j);
            Lap_new(m,j) = 0;
        end
        
        %Iteration
        if abs(Lap_new(int32(m/2), 2) - Lap_old(int32(m/2), 2)) < err && abs(Lap_new(int32(m/2), n-2) - Lap_old(int32(m/2), n-2)) < err && abs(Lap_new(2, int32(n/2)) - Lap_old(2, int32(n/2))) < err && abs(Lap_new(m-2, int32(n/2)) - Lap_old(m-2, int32(n/2))) < err
            Lap_sol = Lap_new;  %Return Lap_new
            fprintf('C_it: %d\n', C_it);
            break;
        else
            Lap_old = Lap_new;  %iteration
        end
    end
end

