%Generate 2D grid for R-Z coordinate system
%Liquid droplet out of capillary

%Import geometry parameters from file
filename = 'parameters.txt';
M = readmatrix(filename);
Para = M(:,1);
disp(Para);

%Read parameters
r_c = Para(1);  %radius of capillary, in m
d = Para(5);  %distance between opening of capillary and the sample, in m

%Dimensionless conversion
d_n = d / r_c;  %distance normalized to radius of capillary, dimensionless
R_max = 100;  %maximum R

%Create file for output
RGrid = fopen('Grid_R.txt','w');
YGrid = fopen('Grid_Y.txt','w');

%Unequal grid
h_R_min = 1e-3;
h_R_e_max = 0.01;
%Generate expanding R grid for electrode from 1 to 0
R = [1];
R_val = 1;
omega_R = 1.05;  %Coefficient for grid expansion in R direction
h = h_R_min;
for i = 1:int32(1 / h_R_min)
    R_val = R_val - h;
    if R_val < 0
        R_val = 0;
        R = [R_val R];
        break;
    else
        R = [R_val R];   
        if h > h_R_e_max
            m_e_max_add = int32(R_val / h_R_e_max);
            h = R_val / double(m_e_max_add);
            R_e_max_add = linspace(0, R_val - h, m_e_max_add);
            R = [R_e_max_add R];
            break;
        else
            h = h * omega_R;
        end
    end
end
m_e = length(R);  %Number of points for electrolyte in R direction.

%Generate expanding grid for the shield
R_val = 1;
h = h_R_min;
h_R_g_max = 3;
for i = 1:int32((R_max - 1) / h_R_min)
    R_val = R_val + h;
    if R_val > R_max
        R_val = R_max;
        R = [R R_val];
        break;
    else
        R = [R R_val];
        if h > h_R_g_max
            disp(R_val);
            disp(h);
            m_g_max_add = int32((R_max - R_val) / h_R_g_max);
            h = (R_max - R_val) / double(m_g_max_add);
            disp(h);
            R_g_max_add = linspace(R_val + h, R_max, m_g_max_add);
            R = [R R_g_max_add];
            break;
        else
            h = h * omega_R;
        end
    end
end
m = length(R);  %Total number of points in R direction
for i = 1:m  %Assign R values
    fprintf(RGrid, '%f\n', R(i));
end
fclose(RGrid);  %Close RGrid before reading it

%Generate Y grid, 0 to 1
n = 51;  %Attention: MUST be odd number
omega_Y = 1.05;  %Coefficient for grid expansion
h_Y_min = 1/100;  %minimum delta_theta, dimensionless
Y = zeros(1, n);  %Create theta vector, Y(1) = 0, dimensionless
while (abs(Y((n+1) / 2) - 0.5) > h_Y_min)  %when theta((n+1) / 2) is too far away from middle point pi/4
    disp(omega_Y);
    h = h_Y_min;
    for j = 2:((n+1) / 2)   
        Y(j) = Y(j-1) + h;
        h = h * omega_Y;
    end
    if (Y((n+1) / 2) > 0.5)
        omega_Y = omega_Y * 0.999;
    else
        omega_Y = omega_Y * 1.001;
    end
end
for j = 1:((n+1) / 2)
    fprintf(YGrid, '%f\n', Y(j));
end
for j = ((n+3) / 2):n
    Y(j) = 1 - Y(n-j+1);
    fprintf(YGrid, '%f\n', Y(j));
end
fprintf('Final omega_theta value: %f\n', omega_Y);
fclose(YGrid);

%Visualize the R-Y grid
R = readmatrix('Grid_R.txt');  %Get R grid from file
m = length(R);
Grid_R = zeros(m,n);
Grid_Y = zeros(m,n);
for i = 1:m
    for j = 1:n
        Grid_R(i,j) = R(i); 
        Grid_Y(i,j) = Y(j); 
    end
end
figure(1)
scatter(Grid_R, Grid_Y)
xlabel('R')
ylabel('Y')

%Visualize the r-z grid, normalized to radius of capillary
Grid_r = zeros(m,n);
Grid_z = zeros(m,n);
for i = 1:m
    for j = 1:n
        Grid_r(i,j) = R(i); %reconvert R-Y to r in r-z coordinates
        Grid_z(i,j) = d_n * Y(j); %reconvert R-Y to z in r-z coordinates
    end
end
figure(2)
scatter(Grid_r, Grid_z)
axis equal
xlabel('Normalized r')
ylabel('Normalized z')

